{-# LANGUAGE TypeApplications #-}
{-# LANGUAGE BangPatterns #-}
{-# LANGUAGE RankNTypes #-}
{-# LANGUAGE NoImplicitPrelude       #-}

{-# LANGUAGE KindSignatures #-}
{-# LANGUAGE GADTs #-}
{-# LANGUAGE ScopedTypeVariables #-}
module Main where

import Data.Set as Set
import Data.String.Conversions (cs)
import System.Environment (getArgs, withArgs)
import Data.Text (Text)
import Data.Vector.Unboxed (Vector)
import Graph.BAC.Clustering
import Graph.BAC.ProxemyOptim
import Graph.BAC.ProxemyOptim
import Graph.FGL
import Graph.Tools.Import
import Graph.Types
import Prelude (String)
import Protolude
import qualified Data.IntMap                       as Dict
import qualified Data.List as List
import qualified Prelude as Prelude
import qualified Data.Graph.Inductive.PatriciaTree as DGIP

import qualified Data.Matrix.Sparse        as SL
import qualified Data.Matrix.Sparse.Static as SLS

import Criterion
import Criterion.Main
import qualified Data.Vector.Sparse.Static as SLV
import Graph.Tools.Random (randomAdjacency)
import Data.Reflection
import qualified Data.Vector.Storable as VS
import qualified Numeric.LinearAlgebra.Static as Dense
import qualified Data.IntMap.Strict as IntMap
import qualified Data.IntSet as IntSet
import qualified Data.Text as Text

setupEnv :: Either String Int -> IO (Dict [Text], Graph () ())
setupEnv (Left fp) = getUnlabGraph (WithFile fp)
setupEnv (Right n) = getUnlabGraph (Random n)

main :: IO ()
main = do
  [fpin, fpout] <- getArgs >>= \xs ->
    if length xs /= 2
    then Prelude.error "Usage: gargantext-graph-exe input_graph.csv output_file.txt"
    else return xs
  setupEnv (Left fpin) >>= \(dico, ~g) -> do
    let (clusts, score) = withG g (\fg -> clusteringOptim 3 Conf fg beta)
        clusts' = Prelude.map (sort . Prelude.map (lkp dico) . IntSet.toList)
                $ sortBy (\a b -> flipOrd $ comparing IntSet.size a b)
                $ Prelude.map (\(n, ns) -> IntSet.insert n ns)
                $ IntMap.toList clusts
    putStrLn $ "#clusters: " ++ show (length clusts')
    putStrLn $ "max cluster size: " ++ show (length (clusts' Prelude.!! 0))
    putStrLn $ "Clustering score: " ++ show score
    withFile fpout WriteMode $ \hndl ->
      forM_ clusts' $ \clust ->
        hPutStrLn hndl $
          "len=" ++ show (length clust) ++
          " [" ++ intercalate ", " [ "'" ++ Text.unpack w ++ "'" | [w] <- clust ] ++ "]\n"

  where beta = 0.0
        flipOrd LT = GT
        flipOrd GT = LT
        flipOrd EQ = EQ
        lkp dico i = fromMaybe (Prelude.error "Node not in dictionary?!") $
          IntMap.lookup i dico
